! ******************************************************************
! ******************************************************************

subroutine CondG(n,set,x,y,w,m,l,u,equatn,linear,jcnnzmax,hnnzmax,&
                 gama,theta,lambda,erropt,outiter,info)
                 
	implicit none
	
	! SCALARS ARGUMENTS
	integer,intent(in)  :: n,m,set,erropt,jcnnzmax,hnnzmax
	integer,intent(out) :: outiter,info
	real(kind=8),intent(in) :: gama,theta,lambda
		
	! ARRAYS ARGUMENTS
	real(kind=8),intent(in)  :: x(n),y(n),l(n),u(n)
	real(kind=8),intent(out) :: w(n)
	logical,intent(in) :: equatn(m),linear(m)
	
	! LOCAL SCALARS
	integer      :: maxoutiter,infoALG
	real(kind=8) :: fopt,phi,stp
	
	! LOCAL ARRAYS
	real(kind=8) :: z(n)

		
	! Initialize parameters
		
	maxoutiter = 50000
	
	! Counters
	
	outiter = 0	
	
	! Set initial w
		
	w = x
	
	! Compute z and gopt
	
	call evalz(n,set,y,w,z,m,l,u,equatn,linear,jcnnzmax,hnnzmax,infoALG)
	
	if ( infoALG /= 0 ) then
		info = -1
		return
	end if
	
	fopt = dot_product( w - y, z - w )
	
	!---------------------------------------------------------------------  
	!     Main loop
	!---------------------------------------------------------------------     

	Main_loop: do
	
		! Test convergence
		
		call evalphi(n,x,y,w,gama,theta,lambda,erropt,phi)
		
		if ( - fopt <= max( phi, 1.0d-8 ) ) then
			info = 0
			return
		end if
		
		! Test whether the number of iterations is exhausted
		
		if ( outiter >= maxoutiter ) then
			info = 1
			return
		end if
		
		!-------------------------------------------------------------------
		!     Iteration
		!-------------------------------------------------------------------
		
		! Increment outiter
				
		outiter = outiter + 1
		
		! Compute alpha
		
		stp = min( 1.0d0 , - fopt / norm2( z - w ) ** 2 )
		
		! Update w
		
		w = w + stp * ( z - w )
		
		! Compute z and fopt
	
		call evalz(n,set,y,w,z,m,l,u,equatn,linear,jcnnzmax,hnnzmax,infoALG)
		
		if ( infoALG /= 0 ) then
			info = -1
			return
		end if
		
		fopt = dot_product( w - y, z - w )

	!---------------------------------------------------------------------
	!     Iterate
	!---------------------------------------------------------------------    

	end do Main_loop

	!--------------------------------------------------------------------- 
	!     End of main loop
	!---------------------------------------------------------------------

end subroutine CondG

! ******************************************************************
! ******************************************************************

subroutine evalphi(n,x,y,w,gama,theta,lambda,erropt,phi)

	implicit none
	
	! SCALARS ARGUMENTS
	integer,intent(in)  :: n,erropt
	real(kind=8),intent(in)  :: gama,theta,lambda
	real(kind=8),intent(out) :: phi
		
	! ARRAYS ARGUMENTS
	real(kind=8),intent(in)  :: x(n),y(n),w(n)
	
	if ( erropt == 1 ) then
		phi = gama * norm2( y - x )**2 + theta * norm2( w - y )**2 + lambda * norm2( w - x )**2
		return
	end if	
	
	if ( erropt == 2 ) then
		phi = norm2( y - x )**2 
		return
	end if	
	
	if ( erropt == 3 ) then
		phi = norm2( w - y )**2
		return
	end if	
	
	if ( erropt == 4 ) then
		phi = norm2( x - w )**2
		return
	end if	
	
end subroutine evalphi
